@@ -2,6 +2,8 @@ import numpy as np
 import os
 import struct
 import random
+import math
+import torch
 
 __all__ = ["event", "loadaerdat", "fromArray", "load"]
 
@@ -200,6 +202,161 @@ class event:
             else:
                 self._removeInvalid(dim_t=resolution)  
 
+    def event_rotation2(self, domains, HW, angle, centers, setscaling=True, drop=True):
+        # domain:{s:空间, xt:x方向时间, yt:y方向时间, t:时间}
+        # HW:高宽
+        # angle:s\xt\yt只需要一个角度，t需要两个角度
+        # center: 输入x、y选择中心
+        xTemp = self.x
+        yTemp = self.y
+        tTemp = self.t
+        pTemp = self.p
+        Points = {'x':xTemp,'y':yTemp,'t':tTemp+1,'p':pTemp}
+        # calculate center of axis
+        HWT = {'x':HW[1],'y':HW[0],'t':max(self.t)+1}
+        center = {}
+        for domain in domains:
+            center[domain] = centers[domain] * HWT[domain]
+        if(setscaling):
+            scaling1 = HWT[domains[0]]/HWT[domains[1]]
+            scaling2 = HWT[domains[1]]/HWT[domains[0]]
+        else:
+            scaling1 = 1
+            scaling2 = 1
+        # Rotation Matrix
+        rotationMatrix = np.array([[1,0,0],[0,1,0],[-center[domains[0]],-center[domains[1]],1]])
+        rotationMatrix = rotationMatrix.dot(np.array([[np.cos(angle),scaling2*np.sin(angle),0],[-scaling1*np.sin(angle),np.cos(angle),0],[0,0,1]]))
+        rotationMatrix = rotationMatrix.dot(np.array([[1,0,0],[0,1,0],[center[domains[0]],center[domains[1]],1]]))
+        rotationMatrix = torch.tensor(rotationMatrix)
+        # Point Matrix
+        oldPointMatrix = np.ones([len(self.x),3])
+        oldPointMatrix[:,0] = Points[domains[0]]
+        oldPointMatrix[:,1] = Points[domains[1]]
+        oldPointMatrix = torch.tensor(oldPointMatrix)
+        # Rotation
+        newPointMatrix = torch.mm(oldPointMatrix, rotationMatrix).numpy()
+        # Remove Invalid
+        Points[domains[0]] = newPointMatrix[:,0].astype(np.int)-1
+        Points[domains[1]] = newPointMatrix[:,1].astype(np.int)-1
+        for domain in domains:
+            if domain is 'x':
+                self.x = Points[domain]
+            elif domain is 'y':
+                self.y = Points[domain]
+            elif domain is 't':
+                self.y = Points[domain]
+            elif domain is 'p': # useless
+                self.p = Points[domain]
+            else:
+                self.p = pTemp
+                self.x = xTemp
+                self.y = yTemp
+                self.t = tTemp
+        if(drop):
+            self._removeInvalid(dim_x=HWT['x'], dim_y=HWT['y'], dim_t=HWT['t'])
+            if len(self.x) == 0:  # avoid dropping all the events
+                self.p = pTemp
+                self.x = xTemp
+                self.y = yTemp
+                self.t = tTemp
+        else:
+            self.x = self.x % HWT['x']
+            self.y = self.x % HWT['y']
+            self.t = self.x % HWT['t']
+        
+    def event_rotation(self, domain, HW, angle, center, drop=False):
+        # domain:{s:空间, xt:x方向时间, yt:y方向时间, t:时间}
+        # HW:高宽
+        # angle:s\xt\yt只需要一个角度，t需要两个角度
+        # center: 输入x、y选择中心
+        x = self.x
+        y = self.y
+        t = self.t
+        p = self.p
+        center_x = center*HW[1] # x轴旋转中心
+        center_y = center*HW[0] # y轴旋转中心
+        resolution = max(self.t)+1 # t轴长度
+        if domain is 's':
+            # Rotation Matrix
+            rotationMatrix = np.array([[1,0,0],[0,1,0],[-center_x,-center_y,1]])
+            rotationMatrix = rotationMatrix.dot(np.array([[np.cos(angle),np.sin(angle),0],[-np.sin(angle),np.cos(angle),0],[0,0,1]]))
+            rotationMatrix = rotationMatrix.dot(np.array([[1,0,0],[0,1,0],[center_x,center_y,1]]))
+            rotationMatrix = torch.tensor(rotationMatrix)
+            # Point Matrix
+            tempMatrix = np.ones([len(self.x),3])
+            tempMatrix[:,0] = self.x
+            tempMatrix[:,1] = self.y
+            tempMatrix = torch.tensor(tempMatrix)
+            # Rotation
+            # newPoint = tempMatrix.dot(rotationMatrix)
+            newPoint = torch.mm(tempMatrix, rotationMatrix).numpy()
+            self.x = newPoint[:,0].astype(np.int)-1
+            self.y = newPoint[:,1].astype(np.int)-1
+            self._removeInvalid(dim_x=HW[1], dim_y=HW[0])
+            if len(self.x) == 0:  # avoid dropping all the events
+                self.p = p
+                self.x = x
+                self.y = y
+                self.t = t
+        elif domain is 'xt' and angle>-math.pi/2 and angle<math.pi/2:
+            self.t = self.t - (self.x-center_x)*np.tan(angle)
+        elif domain is 'yt' and angle>-math.pi/2 and angle<math.pi/2:
+            self.t = self.t - (self.y-center_y)*np.tan(angle)
+        elif domain is 't'and angle[0]>-math.pi/2 and angle[0]<math.pi/2 and angle[1]>-math.pi/2 and angle[1]<math.pi/2:
+            self.t = self.t - (self.x-center_x)*np.tan(angle[0]) - (self.y-center_y)*np.tan(angle[1])
+        elif domain is 'realxt' and angle>-math.pi/4 and angle<math.pi/4:
+            self.t = self.t + (self.x-center_x)*np.sin(angle)*(resolution/HW[1])
+            self.x = (self.x-center_x)*np.cos(angle) + center_x
+        elif domain is 'realyt' and angle>-math.pi/4 and angle<math.pi/4:
+            self.t = self.t + (self.y-center_y)*np.sin(angle)*(resolution/HW[0])
+            self.y = (self.y-center_y)*np.cos(angle) + center_y
+        elif domain is 'realt'and angle[0]>-math.pi/4 and angle[0]<math.pi/4 and angle[1]>-math.pi/4 and angle[1]<math.pi/4:
+            self.t = self.t + (self.x-center_x)*np.sin(angle[0])*(resolution/HW[1]) + (self.y-center_y)*np.sin(angle[1])*(resolution/HW[0])
+            self.x = (self.x-center_x)*np.cos(angle[0]) + center_x
+            self.y = (self.y-center_y)*np.cos(angle[1]) + center_y
+        elif domain is 'mixs':
+            indextsmall = self.t<(max(self.t)/2)
+            indextbig = self.t>(max(self.t)/2)
+            # Rotation Matrix of small
+            rotationMatrixSmall = np.array([[1,0,0],[0,1,0],[-center_x,-center_y,1]])
+            rotationMatrixSmall = rotationMatrixSmall.dot(np.array([[np.cos(angle[0]),np.sin(angle[0]),0],[-np.sin(angle[0]),np.cos(angle[0]),0],[0,0,1]]))
+            rotationMatrixSmall = rotationMatrixSmall.dot(np.array([[1,0,0],[0,1,0],[center_x,center_y,1]]))
+            rotationMatrixSmall = torch.tensor(rotationMatrixSmall)
+            # Rotation Matrix of big
+            rotationMatrixBig = np.array([[1,0,0],[0,1,0],[-center_x,-center_y,1]])
+            rotationMatrixBig = rotationMatrixBig.dot(np.array([[np.cos(angle[1]),np.sin(angle[1]),0],[-np.sin(angle[1]),np.cos(angle[1]),0],[0,0,1]]))
+            rotationMatrixBig = rotationMatrixBig.dot(np.array([[1,0,0],[0,1,0],[center_x,center_y,1]]))
+            rotationMatrixBig = torch.tensor(rotationMatrixBig)
+            # Point Matrix
+            tempMatrix = np.ones([len(self.x),3])
+            tempMatrix[:,0] = self.x
+            tempMatrix[:,1] = self.y
+            tempMatrix = torch.tensor(tempMatrix)
+            # Rotation small
+            newPoint = torch.mm(tempMatrix, rotationMatrixSmall).numpy()
+            self.x[indextsmall] = newPoint[indextsmall,0].astype(np.int)-1
+            self.y[indextsmall] = newPoint[indextsmall,1].astype(np.int)-1
+            # Rotation big
+            newPoint = torch.mm(tempMatrix, rotationMatrixBig).numpy()
+            self.x[indextbig] = newPoint[indextbig,0].astype(np.int)-1
+            self.y[indextbig] = newPoint[indextbig,1].astype(np.int)-1
+            self._removeInvalid(dim_x=HW[1], dim_y=HW[0])
+            if len(self.x) == 0:  # avoid dropping all the events
+                self.p = p
+                self.x = x
+                self.y = y
+                self.t = t
+        if  domain in ('xt','yt','t','realxt','realyt','realt'):
+            if drop == True:
+                self._removeInvalid(dim_x=HW[1], dim_y=HW[0], dim_t=resolution)
+                if len(self.x) == 0:  # avoid dropping all the events
+                    self.p = p
+                    self.x = x
+                    self.y = y
+                    self.t = t
+            else:
+                self.t = self.t % resolution
+
     def event_drop(self, HW):
         x = self.x
         y = self.y
