@@ -317,7 +317,7 @@ class RandomHorizontalFlip(object):
     def __repr__(self):
         return self.__class__.__name__ + '(p={})'.format(self.p)
 
-class EventRotation(object):
+class EventRotation:
     identical = 0
     randomRotation1 = 1
     randomRotation2 = 2
@@ -333,13 +333,14 @@ class EventRotation(object):
     XTRotation = 12
     YTRotation = 13
     XYRotation = 14
-    def __init__(self, HW, methods, drop=False, probabilities=None, number=1, uniform=False):
+    def __init__(self, HW, methods, drop=False, setscaling=True, probabilities=None, number=1, uniform=False):
         number = min(len(methods), number)
         self.number = number
         self.p = probabilities
         self.HW = HW
         self.uniform = uniform
         self.drop = drop
+        self.setscaling = setscaling
         self.method_dict = {EventRotation.identical:['i',[0,0], [0.5,0.5]],
             EventRotation.randomRotation1:['rr1',[-math.pi,math.pi], [0.5,0.5]],
             EventRotation.randomRotation2:['rr2',[-math.pi,math.pi], [0.5,0.5]],
@@ -377,25 +378,25 @@ class EventRotation(object):
             if elem[0] == 'i':
                 return evdata
             elif elem[0] == 'xy':
-                domains = list(np.random.choice(['x','t'], 2, replace=False, p=self.p))
+                domains = list(np.random.choice(['x','y'], 2, replace=False, p=self.p))
                 angle = np.random.uniform(elem[1][0], elem[1][1])
                 centers = {domains[0]:np.random.uniform(elem[2][0], elem[2][1]), domains[1]:np.random.uniform(elem[2][0], elem[2][1])}
-                evdata.event_rotation2(domains, self.HW, angle, centers)
+                evdata.event_rotation2(domains, self.HW, angle, centers, setscaling=self.setscaling, drop=self.drop)
             elif elem[0] == 'xt':
                 domains = list(np.random.choice(['x','t'], 2, replace=False, p=self.p))
                 angle = np.random.uniform(elem[1][0], elem[1][1])
                 centers = {domains[0]:np.random.uniform(elem[2][0], elem[2][1]), domains[1]:np.random.uniform(elem[2][0], elem[2][1])}
-                evdata.event_rotation2(domains, self.HW, angle, centers)
+                evdata.event_rotation2(domains, self.HW, angle, centers, setscaling=self.setscaling, drop=self.drop)
             elif elem[0] == 'yt':
                 domains = list(np.random.choice(['y','t'], 2, replace=False, p=self.p))
                 angle = np.random.uniform(elem[1][0], elem[1][1])
                 centers = {domains[0]:np.random.uniform(elem[2][0], elem[2][1]), domains[1]:np.random.uniform(elem[2][0], elem[2][1])}
-                evdata.event_rotation2(domains, self.HW, angle, centers)
+                evdata.event_rotation2(domains, self.HW, angle, centers, setscaling=self.setscaling, drop=self.drop)
             elif elem[0] == 'rr':
                 domains = list(np.random.choice(['x','y','t'], 2, replace=False, p=self.p))
                 angle = np.random.uniform(elem[1][0], elem[1][1])
                 centers = {domains[0]:np.random.uniform(elem[2][0], elem[2][1]), domains[1]:np.random.uniform(elem[2][0], elem[2][1])}
-                evdata.event_rotation2(domains, self.HW, angle, centers)
+                evdata.event_rotation2(domains, self.HW, angle, centers, setscaling=self.setscaling, drop=self.drop)
             elif elem[0] == 'sr':
                 angle = np.random.uniform(elem[1][0], elem[1][1])
                 center = np.random.uniform(elem[2][0], elem[2][1])
@@ -444,4 +445,5 @@ class EventRotation(object):
     
     def __repr__(self):
         return self.__class__.__name__ + f'\ttransformer times:{self.number}\tHW:{self.HW}\
-            \nmethods:{self.analyse(self.methods)}\ndrop:{self.drop}\nprobabilities:{self.p}'
\ No newline at end of file
+            \nmethods:{self.analyse(self.methods)}\ndrop:{self.drop}\nprobabilities:{self.p}'
+
